@@ -1,10 +1,3 @@
-# Prepare the tests
-file.copy(
-  system.file("its-example.osm.pbf", package = "osmextract"),
-  file.path(tempdir(), "its-example.osm.pbf")
-)
-its_pbf = file.path(tempdir(), "its-example.osm.pbf")
-
 test_that("get_keys (keys): simplest examples work", {
   expect_equal(get_keys('"A"=>"B"'), "A")
   expect_equal(get_keys(c('"A"=>"B"', '"C"=>"D"')), c("A", "C"))
@@ -56,9 +49,25 @@ test_that("get_keys (values): more complicated examples", {
     expected = list(C = c("D", "E")),
     ignore_attr = TRUE
   )
+
+  # there might be empty values or newlines, see
+  # https://github.com/ropensci/osmextract/issues/250
+  expect_identical(
+    object = unclass(get_keys('"A"=>"\n","B"=>"C","D"=>""', values = TRUE)),
+    expected = list(A = "", B = "C", D = ""),
+    ignore_attr = TRUE
+  )
 })
 
 test_that("oe_get_keys: simplest examples work", {
+  withr::local_envvar(
+    .new = list(
+      "OSMEXT_DOWNLOAD_DIRECTORY" = tempdir(),
+      "TESTTHAT" = "true"
+    )
+  )
+  its_pbf = setup_pbf()
+
   # Define path to gpkg object
   its_gpkg = oe_vectortranslate(its_pbf, quiet = TRUE)
 
@@ -70,40 +79,67 @@ test_that("oe_get_keys: simplest examples work", {
   expect_type(keys1, "character")
   expect_type(keys2, "character")
   expect_equal(length(keys1), length(keys2))
-
-  file.remove(its_gpkg)
 })
 
 test_that("oe_get_keys + values: printing method", {
+  withr::local_envvar(
+    .new = list(
+      "OSMEXT_DOWNLOAD_DIRECTORY" = tempdir(),
+      "TESTTHAT" = "true"
+    )
+  )
+  its_pbf = setup_pbf()
+
   expect_snapshot_output(oe_get_keys(its_pbf, values = TRUE))
 
   # Define path to gpkg object
   its_gpkg = oe_vectortranslate(its_pbf, quiet = TRUE)
   expect_snapshot_output(oe_get_keys(its_gpkg, values = TRUE))
-
-  file.remove(its_gpkg)
 })
 
 test_that("oe_get_keys: returns error with wrong inputs", {
   expect_error(
     oe_get_keys(sf::st_sfc(sf::st_point(c(1, 1)), crs = 4326)),
-    "there is no support for objects of class"
+    class = "oe_get_keys-noSupport"
+  )
+  expect_error( # file does not exist
+    oe_get_keys("xxx.gpkg"),
+    class = "oe_get_keys-matchedFileMissing"
+  )
+  expect_error( # length > 1
+    oe_get_keys(c("a.gpkg", "b.gpkg")),
+    class = "oe_get_keys-lengthOneCharacterInput"
   )
-  expect_error(oe_get_keys("xxx.gpkg")) # file does not exist
-  expect_error(oe_get_keys(c("a.gpkg", "b.gpkg"))) # length > 1
 })
 
 test_that("oe_get_keys: reads from sf object", {
-  its_object = oe_read(its_pbf, skip_vectortranslate = TRUE, quiet = TRUE)
-  expect_error(oe_get_keys(its_object), NA)
+  withr::local_envvar(
+    .new = list(
+      "OSMEXT_DOWNLOAD_DIRECTORY" = tempdir(),
+      "TESTTHAT" = "true"
+    )
+  )
+  its_pbf = setup_pbf()
+
+  its = oe_read(its_pbf, skip_vectortranslate = TRUE, quiet = TRUE)
+  expect_error(oe_get_keys(its), NA)
 })
 
 test_that("the output from oe_get_keys is the same as for hstore_get_values", {
+  withr::local_envvar(
+    .new = list(
+      "OSMEXT_DOWNLOAD_DIRECTORY" = tempdir(),
+      "TESTTHAT" = "true"
+    )
+  )
+  its_pbf = setup_pbf()
+
   my_output = oe_get_keys("ITS Leeds", values = TRUE)
   its_leeds_with_surface = oe_get(
     "ITS Leeds",
     query = "SELECT *, hstore_get_value(other_tags, 'surface') AS surface FROM lines",
-    quiet = TRUE
+    quiet = TRUE,
+    force_vectortranslate = TRUE
   )
 
   expect_equal(
@@ -112,46 +148,82 @@ test_that("the output from oe_get_keys is the same as for hstore_get_values", {
   )
 })
 
-
 test_that("oe_get_keys stops when there is no other_tags field", {
+  withr::local_envvar(
+    .new = list(
+      "OSMEXT_DOWNLOAD_DIRECTORY" = tempdir(),
+      "TESTTHAT" = "true"
+    )
+  )
+  its_pbf = setup_pbf()
+
   # Read data ignoring the other_tags field
   its_object = oe_read(
     its_pbf,
-    download_directory = tempdir(),
     query = "SELECT highway, geometry FROM lines",
     quiet = TRUE
   )
   expect_error(
     oe_get_keys(its_object),
-    "The input object must have an other_tags field."
+    class = "oe_get_keys-inputMustHaveOtherTagsField"
   )
 
   # Translate data ignoring the other_tags field
-  its_path = oe_read(
+  its_gpkg = oe_read(
     its_pbf,
     download_only = TRUE,
-    download_directory = tempdir(),
     quiet = TRUE,
     vectortranslate_options = c(
       "-f", "GPKG", "-overwrite", "-select", "highway", "lines"
     )
   )
   expect_error(
-    oe_get_keys(its_path),
-    "The input file must have an other_tags field."
+    oe_get_keys(its_gpkg),
+    class = "oe_get_keys-matchedFileMustHaveOtherTagsFields"
   )
-
-  # Clean tempdir
-  file.remove(its_path)
 })
 
 test_that("oe_get_keys matches input zone with file", {
+  withr::local_envvar(
+    .new = list(
+      "OSMEXT_DOWNLOAD_DIRECTORY" = tempdir(),
+      "TESTTHAT" = "true"
+    )
+  )
+  its_pbf = setup_pbf()
+
   # Simplest example works
   expect_error(oe_get_keys("ITS Leeds"), NA)
 
   # Cannot extract from files that were not previously downloaded
-  expect_error(oe_get_keys("Brazil"))
+  expect_error(oe_get_keys("Brazil"), class = "oe_get_keys-matchedFileMissing")
 })
 
-# Clean tempdir
-file.remove(list.files(tempdir(), pattern = "its-example", full.names = TRUE))
+test_that("oe_get_keys errors when asking for non existing layer", {
+  withr::local_envvar(
+    .new = list(
+      "OSMEXT_DOWNLOAD_DIRECTORY" = tempdir(),
+      "TESTTHAT" = "true"
+    )
+  )
+  its_pbf = setup_pbf()
+  its_gpkg = oe_vectortranslate(its_pbf, quiet = TRUE)
+
+  expect_error(
+    object = oe_get_keys(its_gpkg, layer = "points"),
+    class = "oe_get_keys-missingLayerSelected"
+  )
+})
+
+test_that("oe_get_keys emits warning when some keys were already extracted", {
+  its_pbf = setup_pbf()
+  withr::local_envvar(
+    .new = list("OSMEXT_DOWNLOAD_DIRECTORY" = tempdir())
+  )
+  its_gpkg = oe_vectortranslate(its_pbf, quiet = TRUE, extra_tags = "amenity")
+
+  expect_warning(
+    object = oe_get_keys(its_gpkg),
+    regexp = "The following keys were already extracted"
+  )
+})
